/*---------------------------------------------------------------------------------
Name               : simpleosd.c
Author             : Ondrej Caletka
Description        : Show Keytouch-like OSD. Based on amixer.c from Keytouch project
                     by Marvin Raaijmakers
Date of last change: 10-Sep-2009

    Copyright (C) 2009 Ondrej Caletka
    Copyright (C) 2005-2007 Marvin Raaijmakers

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-----------------------------------------------------------------------------------*/
#define _GNU_SOURCE
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <math.h>

#define UPDATE_INTERVAL		50	/* miliseconds*/
#define SHOW_WINDOW_TIME	1	/* seconds */

#define VOLUME_WINDOW_HEIGHT	25
#define VOLUME_WINDOW_WIDTH	230


typedef struct {
	GtkWidget	*window,
			*volume_bar;
	time_t		close_time;
} VOLUMEBAR_INFO; 



void
create_window (VOLUMEBAR_INFO *volumebar_info)
/*
Input:
	-
Output:
	volumebar_info	- The window element points to the created window and the
			  volume_bar element points to the volume progressbar in the
			  window
Returns:
	-
Description:
	This function creates a window with a progressbar with the following
	properties:
	- It is positioned in the center ot the screen.
	- It has no window decorations and can not be resized by the user.
	- It will allways be above other windows.
	- It is visible on all desktops.
	- It will not be visible in the taskbar an pager.
	- It does not accept focus.
*/
{
	volumebar_info->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position (GTK_WINDOW (volumebar_info->window), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable (GTK_WINDOW (volumebar_info->window), FALSE);
	gtk_window_set_decorated (GTK_WINDOW (volumebar_info->window), FALSE);
	/* The window will allways be above others */
	gtk_window_set_keep_above (GTK_WINDOW (volumebar_info->window), TRUE);
	/* Let the window be visible on all desktops: */
	gtk_window_stick (GTK_WINDOW (volumebar_info->window));
	/* This window will not be visible in the taskbar: */
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (volumebar_info->window), TRUE);
	/* This window will not be visible in the pager: */
	gtk_window_set_skip_pager_hint (GTK_WINDOW (volumebar_info->window), TRUE);
	gtk_window_set_accept_focus (GTK_WINDOW (volumebar_info->window), FALSE);
	
	volumebar_info->volume_bar = gtk_progress_bar_new();
	gtk_widget_show (volumebar_info->volume_bar);
	gtk_container_add (GTK_CONTAINER (volumebar_info->window), volumebar_info->volume_bar);
	gtk_widget_set_size_request (volumebar_info->volume_bar, VOLUME_WINDOW_WIDTH, VOLUME_WINDOW_HEIGHT);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (volumebar_info->volume_bar), 0.52);
	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (volumebar_info->volume_bar), 0.02);
	//gtk_progress_bar_set_text (GTK_PROGRESS_BAR (volumebar_info->volume_bar), "Volume");
}


gboolean
update_window (VOLUMEBAR_INFO *volumebar_info)

/*
Input:
	volumebar_info->close_time	- The time to close the window
	volumebar_info->mix_elem_name	- The name of the mixer element that should
					  be used for retrieving the volume.
Output:
	volumebar_info			- Will be updated
Returns:
	TRUE if this function should be called again after UPDATE_INTERVAL
	miliseconds, otherwise FALSE.
Description:
	This function destroys volumebar_info->window and escapes from the GTK main
	routine if the current time is later than volumebar_info->close_time. If not
	then the volume bar will be updated with the current volume.
*/
{
	gboolean close_window;
	
	/* Check if there is a new message on the queue */
	close_window = (time(NULL) > volumebar_info->close_time);
	if (close_window)
	{
		gtk_widget_destroy (volumebar_info->window);
		gtk_main_quit();
	}
	return !close_window;
}


int
main (int argc, char *argv[])
/*
Input:
	mix_elem_name	- The name of the mixer element that should be used for
			  retrieving the volume.
Description:
	This function creates a window with a volume bar and shows it
	SHOW_WINDOW_TIME seconds when it receives a message on the message queue.
	The key of the message queue is generated by running
	ftok(get_keytouch_user_dir(), MSGQ_AMIXER_PROJ_ID). The messages that are
	sent to this queue should contain the time they are sent. The volume window
	will be showed from the time this function receives the message, until the
	time the message was sent plus SHOW_WINDOW_TIME seconds.
*/
{
	VOLUMEBAR_INFO	volumebar_info;
	gchar *text = "Speaker off";
	gdouble volume = 0.0;
	
	gtk_init (&argc, &argv);

	if (argc > 1)
		text=argv[1];
	if (argc > 2) 
		volume=atof(argv[2])/100;
	if (volume<.00)
		volume=0;
	if (volume>1.0)
		volume=1;

	volumebar_info.close_time = time(NULL) + SHOW_WINDOW_TIME;
	create_window (&volumebar_info);

	gtk_progress_bar_set_text (GTK_PROGRESS_BAR(volumebar_info.volume_bar), text);
	gtk_progress_set_percentage (GTK_PROGRESS(volumebar_info.volume_bar), volume);

	gtk_widget_show (volumebar_info.window);
	g_timeout_add (UPDATE_INTERVAL, (GSourceFunc) update_window, &volumebar_info);
	
	gtk_main();
}


